;	Moore Control
;	File: Remote Volume Cntrl.asm
;
;	Notes:
;8 June 2014 Rev 1.0 first release
;25 Aug 2014 Rev 1.1 cleared mute-on flag when volume raised or lowered from mute on state.
	list	 p=12F1840
	#include p12F1840.inc
	
:Usage:
;In programming mode, measures carrier period and infrared pulse timing from any infrared hand controller. Saves carrier period and number of carrier pulses
;in each carrier on and carrier off code segment. Data saved in EEPROM for each of three commands - volume up, down, and toggle mute.
;In operating mode, compares period and number of carrier pulses in each code segment.  If a match found, sends SPI data to electronic potentiometer to control volume of an 
;external amplifier.   
;	
;   Set configuration bits using definitions from the include file, p16f84.inc
	__config _CONFIG1, _FOSC_INTOSC & _WDTE_SWDTEN & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _IESO_OFF & _FCMEN_OFF
	__config _CONFIG2, _LVP_OFF

;Equivalents
;Flags register:
PgmFlag				equ 	d'0' 
SignalChangedFlag	equ		d'1'
DoneFlag			equ		d'2'
LastInPosFlag		equ		d'3'
MuteOn				equ		d'4'
WaitAfterMute		equ		d'5'	;set when mute toggled to prevent mute state oscillation. 
;Constants
MaxPulseCount		equ		d'80'	;number of memory locations allocated per command.
MaxIdleCount		equ		d'200'	;maximum number IR pulses to detect end of command.
MaxCommands			equ		d'3'	;vol up, vol dwn, toggle mute.
StepSize			equ		d'10'	;Step size for vol up and vol down.

;Registers
	cblock	0x70	;The 16 registers starting here are the same for all memory banks.
	Flags
	Count
	Period
	QtrPeriod
	Tmr1Period
	PulseCount
	CmndCount
	DATA_EE_ADDR
	VolLvl
	temp
	endc

	goto	main


	org	0x04					;interrupt vector here.	
;interrupt called when signal has been low for one period indicating a no IR pulse. 
	clrf	TMR1L				;clear to start counting time till next period.
	movlb	d'0'
	bcf		PIR1,CCP1IF			;clear compare interrupt flag.
	btfss	PORTA,5				;quit routine if signal has just gone neg (IR on.)
	retfie
	btfss	Flags, LastInPosFlag;was signal lo (IR present) before? 
	bra		SignalWasLo
	incf	Count
	movlw	MaxIdleCount		;Check for max time with no IR indicating end of command.	
	subwf	Count,W
	btfsc	STATUS,Z
	bsf		Flags, DoneFlag
	retfie
SignalWasLo						;note that this saves last lo signal count and pos signal counts are saved in running routine.
	bsf		Flags,LastInPosFlag
	call	SaveCount
	incf	Count				;start counting no IR periods
	movlb	d'31'
	movfw	FSR0L				;save FSR0 to shad reg to save on retfie.
	movwf	FSR0L_SHAD
	movfw	FSR0H
	movwf	FSR0H_SHAD
	movlb	d'0'
	retfie
	

main				
;bank 1 registers here.
   	movlb	0x01				;oscon in bank 1
	movlw	b'11110000'			;32mhz int osc using PLL
	movwf	OSCCON				
	movlw	b'00010000'
	movwf	WDTCON				;256 msec WDT timeout, WDT off for now.
;bank 3 registers here.	
	movlb	d'3'
	movlw	d'255'				;Last volume setting in last EEPROM address
	movwf	EEADRL
	BSF 	EECON1, RD 			;EE Read
	movfw	EEDATL
	clrf	ANSELA				;disable all ports digital, none analog.	

;bank 1
	movlb	0x01
	movlw	b'11111000' 		;bit 0 =SDO, 1=SDCLK, 2=CS to potentiometer, 3=always input, 4=PgmButton in and LED out, 5=IR in.
	movwf	TRISA				;note that RA4 starts out as LED output and is switched to input to detect PGM button. 
;bank 5 registers here.
	movlb	d'5'
	movlw	b'00001010'
	movwf	CCP1CON				 ;set TMR1 to compare mode generating software interrupt. 
;bank 4 registers here.
	movlb	d'4'
	movlw	b'00100010'			 ;enable SSP to digital potentiometer, low lvl clk idel, clk is Fosc/64 (.5mhz.)
	movwf	SSPCON
	bsf		SSPSTAT,CKE   		 ;clock phase for SPI data transmission.
;back to bank 0. 
	movlb	0x00
	movlw	b'00000001'			;tmr1 on at 8mhz instruction clock gives 200 counts in 40khz received period signal.
	movwf	T1CON
	clrf	Count
	clrf	Period	
	bsf		INTCON,GIE			;enable interrupts.
	bsf		INTCON,PEIE			;enable peripheral interrupts (for tmr1.) 
	movlw	b'01111011'			;set up tmr2 for 2 secs count w 16 and 64 pre and post scalar and off for now. 
	movwf	T2CON
	bsf		PORTA,2				;make chip select (CS) high to POT to start.
	movlw	0x20				;start data save space at linear indirect memory.
	movwf	FSR0H
	clrf	FSR0L
	clrf	Flags
	clrf	CmndCount		
	movlw	0x80				;on first pwr up, start with half volume.
	movwf	VolLvl
	bsf		PORTA,4				;RA4 always stays hi and is toggled between an output and input for LED on and PGM button detection. d
	movlw	d'47'
	btfsc	STATUS,NOT_TO		;if time out from WDT, skip wait (and skip turning on LED.)
	call	Wait				;Wait routine also turns LED on and off
DetectIR
	movlb	d'1'	
	bcf		WDTCON, SWDTEN		;WDT off.
	movlb	d'0'
	btfsc	PORTA,5				;IR detected?
	bra		CheckPgmButton		;no
	clrf	TMR1L
	clrf	TMR1H
	movlw	0x20
	movwf	FSR0H				;start of linear memory.
	clrf	FSR0L
	movlw	b'00010001'
	andwf	Flags				;Clear all flags except pgm flag and mute on flag.
	clrf	Count
	clrf	PulseCount
	movlb	0x01
	clrWDT
	bsf		WDTCON, SWDTEN		;WDT on. Turned off when pgm returns to idle mode (detect IR.) 
	movlb	0x00
	bra		CheckforHi
CheckPgmButton					;first check to see if LED is on from last received command (on for only 30msec.)
	btfss	T2CON,TMR2ON
	bra		DetectPGMButton		;TMR2 is off and OK to check PGM button which shares this port pin.
	btfss	PIR1,TMR2IF			;if TMR2 is on and OVF, time to turn off LED.
	bra		DetectIR
	bcf		PIR1,TMR2IF
	bcf		T2CON,TMR2ON
	bcf		PORTA,4				;LED OFF and discharge any residual charge so PGM button status is detected.
	nop
	nop
	movlb	d'1'
	bsf		TRISA,4				;turn off LED by making RA4 an input and PGM button detection is enabled.
	movlb	d'0'
	bsf		PORTA,4				;set PORTA latch hi so PGM/LED pin will be high next time LED is turned on.
	bra		DetectIR
DetectPGMButton
	bcf		PORTA,4				;LED OFF and discharge any residual charge so PGM button status is detected.
	movlb	d'1'
	bcf		TRISA,4				;Make RA4 an output to discharge residual charge.
	nop
	nop
	bsf		TRISA,4				;make RA4 an input to detect pgm button.
	movlb	d'0'
	bsf		PORTA,4				;set PORTA latch hi so PGM/LED pin will be high next time LED is turned on.
	btfss	PORTA,4				;now check actual pin status - pgm button down?
	bra		DetectIR
	btfsc	Flags,PgmFlag
	bra		DetectIR
	bsf		Flags,PgmFlag		;set flag for programming mode.
	clrf	CmndCount
	movwf	FSR0H				;start of linear memory.
	clrf	FSR0L
	call	ClearRAM			;clear last IR cmnd data.
	BANKSEL	EEADRL				;
	clrf	EEADRL
	movlw	MaxCommands			;nominally three commands are stored in EEPROM.
	movwf	temp
ClearNextEEPROMSpace	
	call	LoadEEPROM			;now copy cleared RAM into EEPROM to clear EEPROM.
	decfsz	temp
	bra		ClearNextEEPROMSpace
	BANKSEL	EEADRL
	clrf	EEADRL				;all clear, return to start of EEPROM address space.
	movlb	d'0'
	bra		DetectIR
	
CheckforHi	
	btfsc	PORTA,5 			;IR hi or lo?
	bra		$-1
	btfsc	Count,4				;Count = 16?
	bra		ReadTMR1
	btfss	PORTA,5 			;IR now hi?
	bra		$-1
	incf	Count
	bra		CheckforHi
ReadTMR1
	bcf		T1CON,0				;tmr1 off.
	swapf	TMR1H				;routine to divide by 16 to calculate one period.
	swapf	TMR1L
	movlw	b'00001111'
	andwf	TMR1L
	movfw	TMR1H
	addwf	TMR1L
	movfw	TMR1L
	movwi	FSR0++				;save to first memory position and increment pointer.
	movwf	Period
	bcf		STATUS,C			;now calculate qtrperiod.
	rrf		TMR1L
	bcf		STATUS,C
	rrf		TMR1L,0
	movwf	QtrPeriod
	clrf	TMR1H
	clrf	TMR1L
	bsf		T1CON,0				;tmr 1 on
	movlb	d'5'
	movfw	Period	
	movwf	CCPR1L				;move period into tmr1 compare register.
	clrf	CCPR1H
	movlb	d'0'
	bcf		PIR1,CCP1IF			;clear TMR1 compare reg interrupt flag.
	movlb	d'1'
	bsf		PIE1,CCP1IE			;enable TMR1 compare interrupt
	movlb	d'0'
	bcf		Flags,LastInPosFlag	;Current state is with a low signal.
	clrf	Count				;Counter incremented in interrupt routine to count pulses.
;folowing is routine to count carrier pulses.
CheckForIR 						;IR is low signal.
	btfsc	PORTA,5
	bra		CheckForDone
	clrf	TMR1L
	btfsc	Flags,LastInPosFlag	;has the signal been positive (no IR?)
	call	SaveCount 
	incf	Count
	bcf		Flags, LastInPosFlag
CheckForNoIR					;hi is no IR
	btfss	PORTA,5	
	bra		$-1
	clrf	TMR1L				;clear out existing reading and start counting carrier off periods.
	bra		CheckForIR	;
CheckForDone					;done flag set in interrupt routine when no IR detected for many periods. 
	btfss	Flags,DoneFlag
	bra		CheckForIR			;continue counting carrier off IR periods in interrupt period. 
	movlb	0x01
	bcf		WDTCON, SWDTEN		;turn off WDT
	movlb	0x00
	movlb	d'1'
	bcf		PIE1,CCP1IE			;disable TMR1 compare interrupt
	movlb	d'0'
	bcf		Flags,DoneFlag
	bra		PadData				;Move FSR0 pointer to start of next command memory location (needed for pgm mode only.)
SaveCount
	movlw	d'5'				;first eliminate any noise pulses less than 5 carrier cycles long.
	subwf	Count,W
	btfss	STATUS,C
	return
	movlw	MaxPulseCount		;Check for max pulse count.
	subwf	PulseCount,W
	btfsc	STATUS,C
	bra		MaxPulseCountReached
	movfw	Count			
	movwi	FSR0++				;Count is stored in w and moved into linear indirect address space.
	clrf	Count
	incf	PulseCount
	return
MaxPulseCountReached
	clrf	Count
	bsf		Flags, DoneFlag
	return
PadData							;fill all nominally 80 RAM locations.
	movfw	PulseCount	
	sublw	MaxPulseCount
	BTFSC	STATUS,Z
	bra		ProcData
	clrf	INDF0
	incf	FSR0
	incf	PulseCount
	bra		PadData
ProcData						;one IR command has been received for programming or reading a command. 
	;need to reinitialize some values here.
	clrf	PulseCount
	btfss	Flags,PgmFlag
	bra		MatchCommand
	call	LoadEEPROM			;loads next nominally 80 EPROM memory loactions for each command.
	clrf	Count
	clrf	Period
	movlw	d'47'				;1.5 sec wait.
	call	Wait				;don't interpret held down pgm key as IR command key press.
	incf	CmndCount
	movfw	CmndCount
	sublw	MaxCommands
	btfss	STATUS,Z
	goto	DetectIR
	bcf		Flags,PgmFlag
	goto	DetectIR
MatchCommand					;Now match this IR cmnd with a stored command.
	BANKSEL EEADRL ;
	clrf	EEADRL
	clrf	CmndCount
CompareNextCommand	
	movlw	MaxPulseCount		;nominally max of 80 pulses per command.
	movwf	Count
	movlw	0x20
	movwf	FSR0H				;go to start of RAM linear memory space.
	clrf	FSR0L	
	
NextCompare	
	BSF 	EECON1, RD			 ;EE Read
	incf	EEADRL				;prepare for next read
	moviw	FSR0++
	subwf	EEDATL,W
	addlw	d'3'
	btfsc	WREG,7
	bra		TryNextCmnd
	sublw	d'6'
	btfss	STATUS,C
	bra		TryNextCmnd
	decfsz	Count
	bra		NextCompare
	movlb	d'0'	
	call	PerformAction
	goto	DetectIR
TryNextCmnd
	incf	CmndCount
	movfw	CmndCount
	sublw	MaxCommands
	btfsc	STATUS,Z
	bra		WaitforIR			;no match. 
	clrf	EEADRL
	movfw	CmndCount
	movwf	temp
	movlw	MaxPulseCount
CalculateEEADRL	
	addwf	EEADRL
	decfsz	temp	
	bra		CalculateEEADRL
	bra		CompareNextCommand
WaitforIR
	movlb	d'0'
	goto	DetectIR
ClearRAM
	movlw	0x20
	movwf	FSR0H				;go to start of RAM linear memory space.
	clrf	FSR0L
	movlw	d'120'
	movwf	Count
	clrf	WREG
ClearNext
	movwi	FSR0++
	decfsz	Count
	bra		ClearNext
	return
PerformAction
	movlb	d'1'
	bcf		TRISA,4				;turn on LED by making RA4 an output.
	movlb	d'0'
	clrf	TMR2				;LED turns off when TMR2 ovf flag detected in read IR routine. 
	bcf		PIR1, TMR2IF
	bsf		T2CON,TMR2ON
	incf	CmndCount			;need to add 1 for agreement with calling routine.  
	decfsz	CmndCount
	bra		$+2
	bra		VolUp
	decfsz	CmndCount
	bra		$+2	
	bra		VolDwn
	decfsz	CmndCount
	return						;shouldn't end up here.
	bra		ToggleMute
VolUp
	movlw	StepSize
	addwf	VolLvl
	movfw	VolLvl				;store in WREG also for input into SendVolume routine.
	btfss	STATUS,C
	bra		ProcessVolume
	movlw	d'255'				;set at max volume if OVF.
	movwf	VolLvl				;store new lvl in both WREG and VolLvl
	bra		ProcessVolume
VolDwn
	movlw	StepSize
	subwf	VolLvl
	movfw	VolLvl				;store in WREG also for input into SendVolume routine.
	btfsc	STATUS,C
	bra		ProcessVolume
	movlw	d'0'
	movwf	VolLvl				;store new lvl in both WREG and VolLvl
ProcessVolume
	bcf		Flags,MuteOn		;if now in mute state, clear mute on flag.
	call	SendVolume
	return
SendVolume
	movlb	d'0'
	bcf		PIR1,SSP1IF
	bcf		PORTA,2				;chip select low to load data to pot register.
	movlb	d'4'
	movwf	temp				;save vol setting which is in WREG.		
	movfw	SSPBUF				;clear out any received data (junk).
	movlw	b'00010011'			;cmnd to load both channels.
	movwf	SSPBUF
	btfss	SSPSTAT,0			;clear to send.
	bra		$-1
	movlb	d'0'
	btfss	PIR1,SSP1IF
	bra		$-1
	bcf		PIR1,SSP1IF
	movlb	d'4'
	movfw	temp	
	movwf	SSPBUF
	movlb	d'0'
	btfss	PIR1,SSP1IF
	bra		$-1
	bcf		PIR1,SSP1IF
	bsf		PORTA,2				;chip select high to move data to pot.
	btfss	Flags, WaitAfterMute
	return
	movlw	d'30'				;1/2 wait after mute change.
	call	Wait
	bcf		Flags, WaitAfterMute	
	return

ToggleMute
	bsf		Flags,WaitAfterMute	;remember to wait after mute change. 
	btfsc	Flags,MuteOn
	bra		TurnMuteOff	
	bsf		Flags,MuteOn
	clrf	WREG				;set volume to min.
	call	SendVolume
	return
TurnMuteOff
	bcf		Flags,MuteOn
	movfw	VolLvl
	call	SendVolume
	return
LoadEEPROM						;EEADRL is already loaded with starting adress of next command.
	BANKSEL	EEADRL
	movlw	0x20
	movwf	FSR0H				;go to start of RAM linear memory space.
	clrf	FSR0L
	movlw	MaxPulseCount		;nominally 80.
	movwf	Count	
NextData	
	MOVIW	FSR0++			
	MOVWF 	EEDATL 				;Data Memory Value to write
	BCF 	EECON1, CFGS 		;Deselect Configuration space
	BCF 	EECON1, EEPGD 		;Point to DATA memory
	BSF 	EECON1, WREN 		;Enable writes
	BCF 	INTCON, GIE 		;Disable INTs.
	MOVLW 	55h ;
	MOVWF 	EECON2				;Write 55h
	MOVLW 	0AAh ;
	MOVWF 	EECON2 				;Write AAh
	BSF 	EECON1, WR 			;Set WR bit to begin write
	BSF 	INTCON, GIE 		;Enable Interrupts
	BCF 	EECON1, WREN 		;Disable writes
	BTFSC 	EECON1, WR 			;Wait for write to complete
	bra		$-1					;Done
	incf	EEADRL
	decfsz	Count
	bra		NextData
	movlb	d'0'
	return
Wait							;WREG contains wait period. 46 = 1.5sec.  Also turns LED on and off.
	bsf		T2CON,TMR2ON
	movlb	d'1'
	bcf		TRISA,4				;turn on LED by making RA4 an output (output is set high at initialization.)
	movlb	d'0'
	movwf	temp
	clrf	TMR2
ContinueWait	
	bcf		PIR1, TMR2IF
	btfss	PIR1, TMR2IF
	bra		$-1
	decfsz	temp
	bra		ContinueWait
	movlb	d'1'
	bsf		TRISA,4				;turn off LED by making RA4 an input enabling PGM button press detection.
	movlb	d'0'
	bcf		PIR1,TMR2IF
	bcf		T2CON,TMR2ON
	return
	end
